searchData={"items":[{"type":"module","title":"ExUnitProperties","doc":"Provides macros for property-based testing.\n\nThis module provides a few macros that can be used for property-based testing. The core is `check/3`,\nwhich allows executing arbitrary tests on many pieces of generated data. Another one is\n`property/3`, which is meant as a utility to replace the `ExUnit.Case.test/3` macro when writing\nproperties. The last one is `gen/3`, which can be used as syntactic sugar to build generators\n(see `StreamData` for other ways of building generators and for core generators).","ref":"ExUnitProperties.html"},{"type":"module","title":"Overview of property-based testing - ExUnitProperties","doc":"One of the most common ways of writing tests (in Elixir and many other\nlanguages) is to write tests by hand. For example, say that we want to write a\n`starts_with?/2` function that takes two binaries and returns `true` if the\nfirst starts with the second and `false` otherwise. We would likely test such\nfunction with something like this:\n\n    test \"starts_with?/2\" do\n      assert starts_with?(\"foo\", \"f\")\n      refute starts_with?(\"foo\", \"b\")\n      assert starts_with?(\"foo\", \"\")\n      assert starts_with?(\"\", \"\")\n      refute starts_with?(\"\", \"something\")\n    end\n\nThis test highlights the method used to write such kind of tests: they're\nwritten by hand. The process usually consists of testing an expected output on\na set of expected inputs. This works especially well for edge cases, but the\nrobustness of this test could be improved. This is what property-based testing aims\nto solve. Property testing is based on two ideas:\n\n  * specify a set of **properties** that a piece of code should satisfy\n  * test those properties on a very large number of randomly generated data\n\nThe point of specifying **properties** instead of testing manual scenarios is\nthat properties should hold for all the data that the piece of code should be\nable to deal with, and in turn, this plays well with generating data at\nrandom. Writing properties has the added benefit of forcing the programmer to\nthink about their code differently: they have to think about which are\ninvariant properties that their code satisfies.\n\nTo go back to the `starts_with?/2` example above, let's come up with a\nproperty that this function should hold. Since we know that the `Kernel.<>/2`\noperator concatenates two binaries, we can say that a property of\n`starts_with?/2` is that the concatenation of binaries `a` and `b` always\nstarts with `a`. This is easy to model as a property using the `check/3` macro\nfrom this module and generators taken from the `StreamData` module:\n\n    test \"starts_with?/2\" do\n      check all a <- StreamData.binary(),\n                b <- StreamData.binary() do\n        assert starts_with?(a <> b, a)\n      end\n    end\n\nWhen run, this piece of code will generate a random binary and assign it to\n`a`, do the same for `b`, and then run the assertion. This step will be\nrepeated for a large number of times (`100` by default, but it's\nconfigurable), hence generating many combinations of random `a` and `b`. If\nthe body passes for all the generated data, then we consider the property to\nhold. If a combination of randomly generated terms fails the body of the\nproperty, then `ExUnitProperties` tries to find the smallest set of random\ngenerated terms that still fails the property and reports that; this step is\ncalled shrinking.","ref":"ExUnitProperties.html#module-overview-of-property-based-testing"},{"type":"module","title":"Shrinking - ExUnitProperties","doc":"Say that our `starts_with?/2` function blindly returns false when the second\nargument is the empty binary (such as `starts_with?(\"foo\", \"\")`). It's likely\nthat in 100 runs an empty binary will be generated and bound to `b`. When that\nhappens, the body of the property fails but `a` is a randomly generated binary\nand this might be inconvenient: for example, `a` could be `<<0, 74, 192, 99,\n24, 26>>`. In this case, the `check/3` macro tries to **shrink** `a` to the\nsmallest term that still fails the property (`b` is not shrunk because `\"\"` is\nthe smallest binary possible). Doing so will lead to `a = \"\"` and `b = \"\"`\nwhich is the \"minimal\" failing case for our function.\n\nThe example above is a contrived example but shrinking is a very powerful tool\nthat aims at taking the noise out of the failing data.\n\nFor detailed information on shrinking, see also the \"Shrinking\" section in the\ndocumentation for `StreamData`.","ref":"ExUnitProperties.html#module-shrinking"},{"type":"module","title":"Building structs - ExUnitProperties","doc":"We can use the built-in generators to generate other kinds of structs. For\nexample, imagine we wanted to test the following function.\n\n    def noon?(~T[12:00:00]), do: true\n    def noon?(_), do: false\n\nWe could generate `%Time{}` structs as follows:\n\n    defp non_noon_generator do\n      gen all time <- valid_time_generator(), time != ~T[12:00:00] do\n        time\n      end\n    end\n\n    defp valid_time_generator do\n      gen all hour <- StreamData.integer(0..23),\n              minute <- StreamData.integer(0..59),\n              second <- StreamData.integer(0..59) do\n        Time.new!(hour, minute, second)\n      end\n    end\n\nand use them in properties:\n\n    describe \"noon?/1\" do\n      test \"returns true for noon\" do\n        assert noon?(~T[12:00:00]) == true\n      end\n\n      property \"returns false for other times\" do\n        check all time <- non_noon_generator() do\n          assert noon?(time) == false\n        end\n      end\n    end","ref":"ExUnitProperties.html#module-building-structs"},{"type":"module","title":"Resources on property-based testing - ExUnitProperties","doc":"There are many resources available online on property-based testing. An interesting\nread is the original paper that introduced QuickCheck, [\"QuickCheck: A\nLightweight Tool for Random Testing of Haskell\nPrograms\"](http://www.cs.tufts.edu/~nr/cs257/archive/john-hughes/quick.pdf), a\nproperty-testing tool for the Haskell programming language. Another very\nuseful resource especially geared towards Erlang and the BEAM is\n[propertesting.com](http://propertesting.com), a website created by Fred\nHebert: it's a great explanation of property-based testing that includes many\nexamples. Fred's website uses an Erlang property-based testing tool called\n[PropEr](https://github.com/manopapad/proper) but many of the things he talks\nabout apply to `ExUnitProperties` as well.","ref":"ExUnitProperties.html#module-resources-on-property-based-testing"},{"type":"macro","title":"ExUnitProperties.__using__/1","doc":"Sets up an `ExUnit.Case` module for property-based testing.","ref":"ExUnitProperties.html#__using__/1"},{"type":"macro","title":"ExUnitProperties.check/1","doc":"Runs tests for a property.\n\nThis macro provides ad hoc syntax to write properties. Let's see a quick\nexample to get a feel of how it works:\n\n    check all int1 <- integer(),\n              int2 <- integer(),\n              int1 > 0 and int2 > 0,\n              sum = int1 + int2 do\n      assert sum > int1\n      assert sum > int2\n    end\n\nEverything between `check all` and `do` is referred to as **clauses**. Clauses\nare used to specify the values to generate in order to test the properties.\nThe actual tests that the properties hold live in the `do` block.\n\nClauses work exactly like they work in the `gen/1` macro.\n\nThe body passed in the `do` block is where you test that the property holds\nfor the generated values. The body is just like the body of a test: use\n`ExUnit.Assertions.assert/2` (and friends) to assert whatever you want.","ref":"ExUnitProperties.html#check/1"},{"type":"macro","title":"Options - ExUnitProperties.check/1","doc":"* `:initial_size` - (non-negative integer) the initial generation size used\n    to start generating values. The generation size is then incremented by `1`\n    on each iteration. See the \"Generation size\" section of the `StreamData`\n    documentation for more information on generation size. Defaults to `1`.\n\n  * `:max_runs` - (non-negative integer) the total number of generations to\n    run. Defaults to `100`.\n\n  * `:max_run_time` - (non-negative integer) the total number of time (in milliseconds)\n    to run a given check for. This is not used by default, so unless a value\n    is given then the length of the test will be determined by `:max_runs`.\n    If both `:max_runs` and `:max_run_time` are given, then the check will finish at\n    whichever comes first, `:max_runs` or `:max_run_time`.\n\n  * `:max_shrinking_steps` - (non-negative integer) the maximum numbers of\n    shrinking steps to perform in case a failing case is found. Defaults to\n    `100`.\n\n  * `:max_generation_size` - (non-negative integer) the maximum generation\n    size to reach. Note that the size is increased by one on each run. By\n    default, the generation size is unbounded.\n\n  * `:initial_seed` - (integer) the initial seed used to drive the random generation.\n    When `check all` is run with the same initial seed more than once, then every time\n    the terms generated by the generators will be the same as all other runs. This is useful\n    when you want to deterministically reproduce a result. However, it's usually better\n    to leave `:initial_seed` to its default value, which is taken from ExUnit's seed: this\n    way, the random generation will follow options like `--seed` used in ExUnit to\n    deterministically reproduce tests.\n\nIt is also possible to set the values for `:initial_size`, `:max_runs`, `:max_run_time`, and\n`:max_shrinking_steps` through your project's config files. This is especially helpful\nin combination with `:max_runs` when you want to run more iterations on your continuous\nintegration platform, but keep your local tests fast:\n\n    # config/test.exs\n    import Config\n\n    config :stream_data,\n      max_runs: if System.get_env(\"CI\"), do: 1_000, else: 50","ref":"ExUnitProperties.html#check/1-options"},{"type":"macro","title":"Examples - ExUnitProperties.check/1","doc":"Check that all values generated by the `StreamData.integer/0` generator are\nintegers:\n\n    check all int <- integer() do\n      assert is_integer(int)\n    end\n\nCheck that `String.starts_with?/2` and `String.ends_with?/2` always hold for\nconcatenated strings:\n\n    check all start <- binary(),\n              finish <- binary(),\n              concat = start <> finish do\n      assert String.starts_with?(concat, start)\n      assert String.ends_with?(concat, finish)\n    end\n\nCheck that `Kernel.in/2` returns `true` when checking if an element taken out\nof a list is in that same list (changing the number of runs):\n\n    check all list <- list_of(integer()),\n              member <- member_of(list),\n              max_runs: 50 do\n      assert member in list\n    end","ref":"ExUnitProperties.html#check/1-examples"},{"type":"macro","title":"Using `check all` in doctests - ExUnitProperties.check/1","doc":"`check all` can be used in doctests. Make sure that the module where you call\n`doctest(MyModule)` calls `use ExUnitProperties`. Then, you can call `check all`\nin your doctests:\n\n    @doc \"\"\"\n    Tells if a term is an integer.\n\n        iex> check all i <- integer() do\n        ...>   assert int?(i)\n        ...> end\n        :ok\n\n    \"\"\"\n    def int?(i), do: is_integer(i)\n\n`check all` always returns `:ok`, so you can use that as the return value of\nthe whole expression.","ref":"ExUnitProperties.html#check/1-using-check-all-in-doctests"},{"type":"macro","title":"ExUnitProperties.gen/1","doc":"Syntactic sugar to create generators.\n\nThis macro provides ad-hoc syntax to write complex generators. Let's see a\nquick example to get a feel of how it works. Say we have a `User` struct:\n\n    defmodule User do\n      defstruct [:name, :email]\n    end\n\nWe can create a generator of users like this:\n\n    email_generator = map({binary(), binary()}, fn {left, right} -> left <> \"@\" <> right end)\n\n    user_generator =\n      gen all name <- binary(),\n              email <- email_generator do\n        %User{name: name, email: email}\n      end\n\nEverything between `gen all` and `do` is referred to as **clauses**. You can write\nclauses to specify the values to generate. You can then use those values in the `do` body.\nThe newly-created generator will generate values that are the return value of the\n`do` body using the generated values in the clauses.","ref":"ExUnitProperties.html#gen/1"},{"type":"macro","title":"Clauses - ExUnitProperties.gen/1","doc":"As seen in the example above, clauses can be of the following types:\n\n  * **value generation** - they have the form `pattern <- generator` where `generator` must be a\n    generator. These clauses take a value out of `generator` on each run and match it against\n    `pattern`. Variables bound in `pattern` can be then used throughout subsequent clauses and\n    in the `do` body. If `pattern` doesn't match a generated value, it's treated like a filter\n    (see the \"filtering\" clauses described below).\n\n  * **filtering and binding** - they have the form `expression`. If a filtering clause returns\n    a truthy value, then the set of generated values that appear before the\n    filtering clause is considered valid and generation continues. If the\n    filtering clause returns a falsey value, then the current value is\n    considered invalid and a new value is generated. Note that filtering\n    clauses should not filter out too many times; in case they do, a\n    `StreamData.FilterTooNarrowError` error is raised (same as `StreamData.filter/3`).\n    Filtering clauses can be used also to assign variables: for example, `a = :foo` is a valid\n    clause.\n\nThe behaviour of the clauses above is similar to the behaviour of clauses in\n`Kernel.SpecialForms.for/1`.","ref":"ExUnitProperties.html#gen/1-clauses"},{"type":"macro","title":"Body - ExUnitProperties.gen/1","doc":"The return value of the body passed in the `do` block is what is ultimately\ngenerated by the generator return by this macro.","ref":"ExUnitProperties.html#gen/1-body"},{"type":"macro","title":"Shrinking - ExUnitProperties.gen/1","doc":"See the module documentation for more information on shrinking. Clauses affect\nshrinking in the following way:\n\n  * filtering clauses affect shrinking like `StreamData.filter/3`\n  * value generation clauses affect shrinking similarly to `StreamData.bind/2`","ref":"ExUnitProperties.html#gen/1-shrinking"},{"type":"function","title":"ExUnitProperties.pick/1","doc":"Picks a random element generated by the `StreamData` generator `data`.\n\nThis function uses the current ExUnit seed to generate a random term from `data`. The generation\nsize (see [*Generation size*](StreamData.html#module-generation-size)) is chosen at random between in `1..100`. If you want finer\ncontrol over the generation size, you can use functions like `StreamData.resize/2` to resize\n`data` or `StreamData.scale/2` to scale the generation size.","ref":"ExUnitProperties.html#pick/1"},{"type":"function","title":"Examples - ExUnitProperties.pick/1","doc":"ExUnitProperties.pick(StreamData.integer())\n    #=> -21","ref":"ExUnitProperties.html#pick/1-examples"},{"type":"macro","title":"ExUnitProperties.property/1","doc":"Defines a not-implemented property test with a string.\n\nProvides a convenient macro that allows a property test to be defined with a\nstring, but not yet implemented. The resulting property test will always\nfail and print a \"Not implemented\" error message. The resulting test case is\nalso tagged with `:not_implemented`.\n\nThis behavior is similar to `ExUnit.Case.test/1`.","ref":"ExUnitProperties.html#property/1"},{"type":"macro","title":"Examples - ExUnitProperties.property/1","doc":"property \"this will be a property test in the future\"","ref":"ExUnitProperties.html#property/1-examples"},{"type":"macro","title":"ExUnitProperties.property/3","doc":"Defines a property and imports property-testing facilities in the body.\n\nThis macro is similar to `ExUnit.Case.test/3`, except that it denotes a\n**property**. In the given body, all the functions exposed by `StreamData` are\nimported, as well as `check/2`.\n\nWhen defining a test whose body only consists of one or more `check/2` calls,\nit's advised to use `property/3` so as to clearly denote and scope properties.\nDoing so will also improve reporting.","ref":"ExUnitProperties.html#property/3"},{"type":"macro","title":"Examples - ExUnitProperties.property/3","doc":"use ExUnitProperties\n\n    property \"reversing a list doesn't change its length\" do\n      check all list <- list_of(integer()) do\n        assert length(list) == length(:lists.reverse(list))\n      end\n    end","ref":"ExUnitProperties.html#property/3-examples"},{"type":"module","title":"StreamData","doc":"Functions to create and combine generators.\n\nA generator is a `StreamData` struct. Generators can be created through the\nfunctions exposed in this module, like `constant/1`, and by combining other\ngenerators through functions like `bind/2`.\n\nSimilar to the `Stream` module, the functions in this module return a lazy\nconstruct. We can get values out of a generator by enumerating the generator.\nGenerators always generate an infinite stream of values (which are randomized\nmost of the time).\n\nFor example, to get an infinite stream of integers that starts with small\nintegers and progressively grows the boundaries, you can use `integer/0`:\n\n    Enum.take(StreamData.integer(), 10)\n    #=> [-1, 0, -3, 4, -4, 5, -1, -3, 5, 8]\n\nAs you can see above, values emitted by a generator are not unique.\n\nIn many applications of generators, the longer the generator runs the larger\nthe generated values will be. For integers, a larger integer means a bigger number.\nFor lists, it may mean a list with more elements. This is controlled by a parameter\nthat we call the **generation size** (see the \"Generation size\" section below).\n\nStreamData is often used to generate random values. It is also the foundation\nfor property-based testing. See `ExUnitProperties` for more information.","ref":"StreamData.html"},{"type":"module","title":"Enumeration - StreamData","doc":"Generators implement the `Enumerable` protocol. The enumeration starts with a\nsmall generation size, which increases when the enumeration continues (up to a\nfixed maximum size).\n\nSince generators are proper streams, functions from the `Stream` module can be\nused to stream values out of them. For example, to build an infinite stream of\npositive even integers, you can do:\n\n    StreamData.integer()\n    |> Stream.filter(& &1 > 0)\n    |> Stream.map(& &1 * 2)\n    |> Enum.take(10)\n    #=> [4, 6, 4, 10, 14, 16, 4, 16, 36, 16]\n\nGenerators that are manipulated via the `Stream` and `Enum` modules are no\nlonger **shrinkable** (see the section about shrinking below). If you want\ngeneration through the `Enumerable` protocol to be reproducible, see `seeded/2`.","ref":"StreamData.html#module-enumeration"},{"type":"module","title":"Generation size - StreamData","doc":"Generators have access to a generation parameter called the **generation\nsize**, which is a non-negative integer. This parameter is meant to bind the\ndata generated by each generator in a way that is completely up to the\ngenerator. For example, a generator that generates integer can use the `size`\nparameter to generate integers inside the `-size..size` range. In a similar\nway, a generator that generates lists could use this parameter to generate a\nlist with `0` to `size` elements. During composition, it is common for the\n\"parent generator\" to pass the size to the composed generators.\n\nWhen creating generators, they can access the generation size using the\n`sized/1` function. Generators can be resized to a fixed generation size using\n`resize/2`.","ref":"StreamData.html#module-generation-size"},{"type":"module","title":"Shrinking - StreamData","doc":"`StreamData` generators are also shrinkable. The idea behind shrinking is\nto find the simplest value that respects a certain condition. For example,\nduring property-based tests, we use shrinking to find the integer closest to 0\nor the smallest list that makes a test fail. By reporting the simplest data\nstructure that triggers an error, the failure becomes easier to understand\nand reproduce.\n\nEach generator has its own logic to shrink values. Those are outlined in each\ngenerator documentation.\n\nNote that the generation size is not related in any way to shrinking: while\nintuitively one may think that shrinking just means decreasing the generation\nsize, in reality the shrinking rule is bound to each generated value. One way\nto look at it is that shrinking a list is always the same, regardless of its\ngenerated length.","ref":"StreamData.html#module-shrinking"},{"type":"module","title":"Special generators - StreamData","doc":"Some Elixir types are implicitly converted to `StreamData` generators when\ncomposed or used in property-based testing. These types are:\n\n  * atoms - they generate themselves. For example, `:foo` is equivalent to\n    `StreamData.constant(:foo)`.\n\n  * tuples of generators - they generate tuples where each value is a value\n    generated by the corresponding generator, exactly like described in\n    `tuple/1`. For example, `{StreamData.integer(), StreamData.boolean()}`\n    generates entries like `{10, false}`.\n\nNote that *these terms must be explicitly converted to StreamData generators*.\nThis means that these terms are not full-fledged generators. For example, atoms\ncannot be enumerated directly as they don't implement the `Enumerable` protocol.\nHowever, `StreamData.constant(:foo)` is enumerable as it has been wrapped in\na `StreamData` function.","ref":"StreamData.html#module-special-generators"},{"type":"function","title":"StreamData.atom/1","doc":"Generates atoms of various `kind`s.\n\n`kind` can be:\n\n  * `:alphanumeric` - this generates alphanumeric atoms that don't need to be quoted when\n    written as literals. For example, it will generate `:foo` but not `:\"foo bar\"`.\n\n  * `:alias` - generates Elixir aliases like `Foo` or `Foo.Bar.Baz`.\n\nThese are some of the most common kinds of atoms usually used in Elixir applications. If you\nneed completely arbitrary atoms, you can use a combination of `map/2`, `String.to_atom/1`,\nand string-focused generators to transform arbitrary strings into atoms:\n\n    printable_atom =\n      StreamData.map(\n        StreamData.string(:printable, max_length: 255),\n        &String.to_atom/1\n      )\n\nBear in mind the [system limit](http://erlang.org/doc/efficiency_guide/advanced.html#system-limits)\nof 255 characters in an atom when doing so.","ref":"StreamData.html#atom/1"},{"type":"function","title":"Examples - StreamData.atom/1","doc":"Enum.take(StreamData.atom(:alphanumeric), 3)\n    #=> [:xF, :y, :B_]","ref":"StreamData.html#atom/1-examples"},{"type":"function","title":"Shrinking - StreamData.atom/1","doc":"Shrinks towards smaller atoms and towards \"simpler\" letters (like towards only alphabet\nletters).","ref":"StreamData.html#atom/1-shrinking"},{"type":"function","title":"StreamData.binary/1","doc":"Generates binaries.\n\nThe length of the generated binaries is limited by the generation size.","ref":"StreamData.html#binary/1"},{"type":"function","title":"Options - StreamData.binary/1","doc":"* `:length` - (non-negative integer) sets the exact length of the generated\n    binaries (same as in `list_of/2`).\n\n  * `:min_length` - (non-negative integer) sets the minimum length of the\n    generated binaries (same as in `list_of/2`). Ignored if `:length` is\n    present.\n\n  * `:max_length` - (non-negative integer) sets the maximum length of the\n    generated binaries (same as in `list_of/2`). Ignored if `:length` is\n    present.","ref":"StreamData.html#binary/1-options"},{"type":"function","title":"Examples - StreamData.binary/1","doc":"Enum.take(StreamData.binary(), 3)\n    #=> [<<1>>, \"\", \"@Q\"]","ref":"StreamData.html#binary/1-examples"},{"type":"function","title":"Shrinking - StreamData.binary/1","doc":"Values generated by this generator shrink by becoming smaller binaries and by\nhaving individual bytes that shrink towards `0`.","ref":"StreamData.html#binary/1-shrinking"},{"type":"function","title":"StreamData.bind/2","doc":"Binds each element generated by `data` to a new generator returned by applying `fun`.\n\nThis function is the basic mechanism for composing generators. It takes a\ngenerator `data` and invokes `fun` with each element in `data`. `fun` must\nreturn a new *generator* that is effectively used to generate items from\nnow on.","ref":"StreamData.html#bind/2"},{"type":"function","title":"Examples - StreamData.bind/2","doc":"Say we wanted to create a generator that returns two-element tuples where the\nfirst element is a non-empty list, and the second element is a random element from that\nlist. To do that, we can first generate a list and then bind a function to\nthat list; this function will return the list and a random element from it.\n\n    StreamData.bind(StreamData.list_of(StreamData.integer(), min_length: 1), fn list ->\n      StreamData.bind(StreamData.member_of(list), fn elem ->\n        StreamData.constant({list, elem})\n      end)\n    end)","ref":"StreamData.html#bind/2-examples"},{"type":"function","title":"Shrinking - StreamData.bind/2","doc":"The generator returned by `bind/2` shrinks by first shrinking the value\ngenerated by the inner generator and then by shrinking the outer generator\ngiven as `data`. When `data` shrinks, `fun` is once more applied on the\nshrunk value and returns a whole new generator, which will most likely\nemit new items.","ref":"StreamData.html#bind/2-shrinking"},{"type":"function","title":"StreamData.bind_filter/3","doc":"Binds each element generated by `data` and to a new generator returned by\napplying `fun` or filters the generated element.\n\nWorks similarly to `bind/2` but allows to filter out unwanted values. It takes\na generator `data` and invokes `fun` with each element generated by `data`.\n`fun` must return one of:\n\n  * `{:cont, generator}` - `generator` is then used to generate the next\n    element\n\n  * `:skip` - the value generated by `data` is filtered out and a new element\n    is generated\n\nSince this function acts as a filter as well, it behaves similarly to\n`filter/3`: when more than `max_consecutive_failures` elements are filtered\nout (that is, `fun` returns `:skip`), a `StreamData.FilterTooNarrowError` is\nraised. See the documentation for `filter/3` for suggestions on how to avoid\nsuch errors.\n\nThe function can accept one or two arguments. If a two-argument function is\npassed, the second argument will be the number of tries left before raising\n`StreamData.FilterTooNarrowError`.","ref":"StreamData.html#bind_filter/3"},{"type":"function","title":"Examples - StreamData.bind_filter/3","doc":"Say we wanted to create a generator that generates two-element tuples where\nthe first element is a list of integers with an even number of members and the\nsecond element is a member of that list. We can do that by generating a list\nand, if it has even length, taking an element out of it, otherwise filtering\nit out.\n\n    require Integer\n\n    list_data = StreamData.list_of(StreamData.integer(), min_length: 1)\n\n    data =\n      StreamData.bind_filter(list_data, fn\n        list when Integer.is_even(length(list)) ->\n          inner_data = StreamData.bind(StreamData.member_of(list), fn member ->\n            StreamData.constant({list, member})\n          end)\n          {:cont, inner_data}\n        _odd_list ->\n          :skip\n      end)\n\n    Enum.at(data, 0)\n    #=> {[-6, -7, -4, 5, -9, 8, 7, -9], 5}","ref":"StreamData.html#bind_filter/3-examples"},{"type":"function","title":"Shrinking - StreamData.bind_filter/3","doc":"This generator shrinks like `bind/2` but values that are skipped are not used\nfor shrinking (similarly to how `filter/3` works).","ref":"StreamData.html#bind_filter/3-shrinking"},{"type":"function","title":"StreamData.bitstring/1","doc":"Generates bitstrings.\n\nThe length of the generated bitstring is limited by the generation size.","ref":"StreamData.html#bitstring/1"},{"type":"function","title":"Options - StreamData.bitstring/1","doc":"* `:length` - (non-negative integer) sets the exact length of the generated\n    bitstrings (same as in `list_of/2`).\n\n  * `:min_length` - (non-negative integer) sets the minimum length of the\n    generated bitstrings (same as in `list_of/2`). Ignored if `:length` is\n    present.\n\n  * `:max_length` - (non-negative integer) sets the maximum length of the\n    generated bitstrings (same as in `list_of/2`). Ignored if `:length` is\n    present.","ref":"StreamData.html#bitstring/1-options"},{"type":"function","title":"Examples - StreamData.bitstring/1","doc":"Enum.take(StreamData.bitstring(), 3)\n    #=> [<<0::size(1)>>, <<2::size(2)>>, <<5::size(3)>>]","ref":"StreamData.html#bitstring/1-examples"},{"type":"function","title":"Shrinking - StreamData.bitstring/1","doc":"Values generated by this generator shrink by becoming smaller bitstrings and\nby having the individual bits go towards `0`.","ref":"StreamData.html#bitstring/1-shrinking"},{"type":"function","title":"StreamData.boolean/0","doc":"Generates boolean values.","ref":"StreamData.html#boolean/0"},{"type":"function","title":"Examples - StreamData.boolean/0","doc":"Enum.take(StreamData.boolean(), 3)\n    #=> [true, true, false]","ref":"StreamData.html#boolean/0-examples"},{"type":"function","title":"Shrinking - StreamData.boolean/0","doc":"Shrinks towards `false`.","ref":"StreamData.html#boolean/0-shrinking"},{"type":"function","title":"StreamData.byte/0","doc":"Generates bytes.\n\nA byte is an integer between `0` and `255`.","ref":"StreamData.html#byte/0"},{"type":"function","title":"Examples - StreamData.byte/0","doc":"Enum.take(StreamData.byte(), 3)\n    #=> [102, 161, 13]","ref":"StreamData.html#byte/0-examples"},{"type":"function","title":"Shrinking - StreamData.byte/0","doc":"Values generated by this generator shrink like integers, so towards bytes\ncloser to `0`.","ref":"StreamData.html#byte/0-shrinking"},{"type":"function","title":"StreamData.chardata/0","doc":"Generates chardata.\n\nChardata are values of the `t:IO.chardata/0` type.","ref":"StreamData.html#chardata/0"},{"type":"function","title":"Examples - StreamData.chardata/0","doc":"Enum.take(StreamData.chardata(), 3)\n    #=> [\"\", [\"\"], [12174]]","ref":"StreamData.html#chardata/0-examples"},{"type":"function","title":"Shrinking - StreamData.chardata/0","doc":"Shrinks towards less nested chardata and ultimately towards smaller binaries.","ref":"StreamData.html#chardata/0-shrinking"},{"type":"function","title":"StreamData.check_all/3","doc":"Checks the behaviour of a given function on values generated by `data`.\n\nThis function takes a generator and a function `fun` and verifies that that\nfunction \"holds\" for all generated data. `fun` is called with each generated\nvalue and can return one of:\n\n  * `{:ok, term}` - means that the function \"holds\" for the given value. `term`\n    can be anything and will be used for internal purposes by `StreamData`.\n\n  * `{:error, term}` - means that the function doesn't hold for the given\n    value. `term` is the term that will be shrunk to find the minimal value\n    for which `fun` doesn't hold. See below for more information on shrinking.\n\nWhen a value is found for which `fun` doesn't hold (returns `{:error, term}`),\n`check_all/3` tries to shrink that value in order to find a minimal value that\nstill doesn't satisfy `fun`.\n\nThe return value of this function is one of:\n\n  * `{:ok, ok_map}` - if all generated values satisfy `fun`. `ok_map` is a map\n    of metadata that contains no keys for now.\n\n  * `{:error, error_map}` - if a generated value doesn't satisfy `fun`.\n    `error_map` is a map of metadata that contains the following keys:\n\n    * `:original_failure` - if `fun` returned `{:error, term}` for a generated\n      value, this key in the map will be `term`.\n\n    * `:shrunk_failure` - the value returned in `{:error, term}` by `fun`\n      when invoked with the smallest failing value that was generated.\n\n    * `:nodes_visited` - the number of nodes (a positive integer) visited in\n      the shrinking tree in order to find the smallest value. See also the\n      `:max_shrinking_steps` option.\n\n    * `:successful_runs` - the number of successful runs before a failing value was found.","ref":"StreamData.html#check_all/3"},{"type":"function","title":"Options - StreamData.check_all/3","doc":"This function takes the following options:\n\n  * `:initial_seed` - three-element tuple with three integers that is used as\n    the initial random seed that drives the random generation. This option is\n    required.\n\n  * `:initial_size` - (non-negative integer) the initial generation size used\n    to start generating values. The generation size is then incremented by `1`\n    on each iteration. See the \"Generation size\" section of the module\n    documentation for more information on generation size. Defaults to `1`.\n\n  * `:max_runs` - (non-negative integer) the total number of elements to\n    generate out of `data` and check through `fun`. Defaults to `100`.\n\n  * `:max_run_time` - (non-negative integer) the total number of time (in milliseconds)\n    to run a given check for. This is not used by default, so unless a value\n    is given, then the length of the check will be determined by `:max_runs`.\n    If both `:max_runs` and `:max_run_time` are given, then the check will finish at\n    whichever comes first, `:max_runs` or `:max_run_time`.\n\n  * `:max_shrinking_steps` - (non-negative integer) the maximum numbers of\n    shrinking steps to perform in case `check_all/3` finds an element that\n    doesn't satisfy `fun`. Defaults to `100`.","ref":"StreamData.html#check_all/3-options"},{"type":"function","title":"Examples - StreamData.check_all/3","doc":"Let's try out a contrived example: we want to verify that the `integer/0`\ngenerator generates integers that are not `0` or multiples of `11`. This\nverification is broken by design because `integer/0` is likely to generate\nmultiples of `11` at some point, but it will show the capabilities of\n`check_all/3`. For the sake of the example, let's say we want the values that\nfail to be represented as strings instead of the original integers that\nfailed. We can implement what we described like this:\n\n    options = [initial_seed: :os.timestamp()]\n\n    {:error, metadata} = StreamData.check_all(StreamData.integer(), options, fn int ->\n      if int == 0 or rem(int, 11) != 0 do\n        {:ok, nil}\n      else\n        {:error, Integer.to_string(int)}\n      end\n    end)\n\n    metadata.nodes_visited\n    #=> 7\n    metadata.original_failure\n    #=> 22\n    metadata.shrunk_failure\n    #=> 11\n\nAs we can see, the function we passed to `check_all/3` \"failed\" for `int =\n22`, and `check_all/3` was able to shrink this value to the smallest failing\nvalue, which in this case is `11`.","ref":"StreamData.html#check_all/3-examples"},{"type":"function","title":"StreamData.codepoint/1","doc":"Generates an integer corresponding to a valid UTF-8 codepoint of the given kind.\n\n`kind` can be:\n\n  * `:ascii` - only ASCII characters are generated. Shrinks towards lower codepoints.\n\n  * `:alphanumeric` - only alphanumeric characters (`?a..?z`, `?A..?Z`, `?0..?9`)\n  are generated. Shrinks towards `?a` following the order shown previously.\n\n  * `:printable` - only printable codepoints\n  (`String.printable?(< >)` returns `true`)\n    are generated. Shrinks towards lower codepoints.\n\n  * `:utf8` - all valid codepoints (`< >)` does not raise)\n    are generated. Shrinks towards lower codepoints.\n\nDefaults to `:utf8`.","ref":"StreamData.html#codepoint/1"},{"type":"function","title":"Examples - StreamData.codepoint/1","doc":"Enum.take(StreamData.codepoint(), 3)\n    #=> [889941, 349615, 1060099]\n\n    Enum.take(StreamData.codepoint(:ascii), 3)\n    #=> ~c\"Kk:\"","ref":"StreamData.html#codepoint/1-examples"},{"type":"function","title":"StreamData.constant/1","doc":"A generator that always generates the given term.","ref":"StreamData.html#constant/1"},{"type":"function","title":"Examples - StreamData.constant/1","doc":"iex> Enum.take(StreamData.constant(:some_term), 3)\n    [:some_term, :some_term, :some_term]","ref":"StreamData.html#constant/1-examples"},{"type":"function","title":"Shrinking - StreamData.constant/1","doc":"This generator doesn't shrink.","ref":"StreamData.html#constant/1-shrinking"},{"type":"function","title":"StreamData.filter/3","doc":"Filters the given generator `data` according to the given `predicate` function.\n\nOnly elements generated by `data` that pass the filter are kept in the\nresulting generator.\n\nIf the filter is too strict, it can happen that too few values generated by `data` satisfy it.\nIn case more than `max_consecutive_failures` consecutive values don't satisfy the filter, a\n`StreamData.FilterTooNarrowError` will be raised. There are a few ways you can avoid risking\n`StreamData.FilterTooNarrowError` errors.\n\n  * Try to make sure that your filter filters out only a small subset of the elements generated\n    by `data`. For example, having something like `StreamData.filter(StreamData.integer(), &(&1 !=\n    0))` is usually fine because only a very tiny part of the generation space (integers) is\n    being filtered out.\n\n  * Keep an eye on how the generation size affects the generator being filtered. For example,\n    take something like `StreamData.filter(StreamData.positive_integer(), &(&1 not in 1..5)`.\n    While it seems like this filter is not that strict (as we're filtering out only a handful of\n    numbers out of all natural numbers), this filter will fail with small generation sizes.\n    Since `positive_integer/0` returns an integer between `0..size`, if `size` is small (for\n    example, less than 10) then the probability of generating many consecutive values in `1..5`\n    is high.\n\n  * Try to restructure your generator so that instead of generating many values and taking out\n    the ones you don't want, you instead generate values and turn all of them into values that\n    are suitable. A good example is a generator for even integers. You could write it as\n\n        def even_integers() do\n          StreamData.filter(StreamData.integer(), &Integer.is_even/1)\n        end\n\n    but this would generate many unused values, increasing likeliness of\n    `StreamData.FilterTooNarrowError` errors and performing inefficiently. Instead, you can use\n    `map/2` to turn all integers into even integers:\n\n        def even_integers() do\n          StreamData.map(StreamData.integer(), &(&1 * 2))\n        end","ref":"StreamData.html#filter/3"},{"type":"function","title":"Shrinking - StreamData.filter/3","doc":"All the values that each generated value shrinks to satisfy `predicate` as\nwell.","ref":"StreamData.html#filter/3-shrinking"},{"type":"function","title":"StreamData.fixed_list/1","doc":"Generates a list of fixed length where each element is generated from the\ncorresponding generator in `data`.","ref":"StreamData.html#fixed_list/1"},{"type":"function","title":"Examples - StreamData.fixed_list/1","doc":"data = StreamData.fixed_list([StreamData.integer(), StreamData.binary()])\n    Enum.take(data, 3)\n    #=> [[1, <<164>>], [2, \".T\"], [1, \"\"]]","ref":"StreamData.html#fixed_list/1-examples"},{"type":"function","title":"Shrinking - StreamData.fixed_list/1","doc":"Shrinks by shrinking each element in the generated list according to the\ncorresponding generator. Shrunk lists never lose elements.","ref":"StreamData.html#fixed_list/1-shrinking"},{"type":"function","title":"StreamData.fixed_map/1","doc":"Generates maps with fixed keys and generated values.\n\n`data_map` is a map or keyword list of `fixed_key => data` pairs. Maps generated by this\ngenerator will have the same keys as `data_map` and values corresponding to values generated by\nthe generator under those keys.\n\nSee also `optional_map/1`.","ref":"StreamData.html#fixed_map/1"},{"type":"function","title":"Examples - StreamData.fixed_map/1","doc":"data = StreamData.fixed_map(%{\n      integer: StreamData.integer(),\n      binary: StreamData.binary(),\n    })\n    Enum.take(data, 3)\n    #=> [%{binary: \"\", integer: 1}, %{binary: \"\", integer: -2}, %{binary: \"R1^\", integer: -3}]","ref":"StreamData.html#fixed_map/1-examples"},{"type":"function","title":"Shrinking - StreamData.fixed_map/1","doc":"This generator shrinks by shrinking the values of the generated map.","ref":"StreamData.html#fixed_map/1-shrinking"},{"type":"function","title":"StreamData.float/1","doc":"Generates floats according to the given `options`.\n\nThe complexity of the generated floats grows proportionally to the generation size.","ref":"StreamData.html#float/1"},{"type":"function","title":"Options - StreamData.float/1","doc":"* `:min` - (float) if present, the generated floats will be greater than or equal to this\n    value.\n\n  * `:max` - (float) if present, the generated floats will be less than or equal to this value.\n\nIf neither of `:min` or `:max` is provided, then unbounded floats will be generated.","ref":"StreamData.html#float/1-options"},{"type":"function","title":"Shrinking - StreamData.float/1","doc":"Values generated by this generator will shrink towards simpler floats. Such values are not\nguaranteed to shrink towards smaller or larger values (but they will never violate the `:min` or\n`:max` options).","ref":"StreamData.html#float/1-shrinking"},{"type":"function","title":"StreamData.frequency/1","doc":"Generates values from different generators with specified probability.\n\n`frequencies` is a list of `{frequency, data}` where `frequency` is an integer\nand `data` is a generator. The resulting generator will generate data from one\nof the generators in `frequency`, with probability `frequency / vsum_of_frequencies`.","ref":"StreamData.html#frequency/1"},{"type":"function","title":"Examples - StreamData.frequency/1","doc":"Let's build a generator that returns a binary around 25% of the time and an\ninteger around 75% of the time. We'll use `integer/0` first so that generated values\nwill shrink towards integers.\n\n    ints_and_some_bins = StreamData.frequency([\n      {3, StreamData.integer()},\n      {1, StreamData.binary()},\n    ])\n    Enum.take(ints_and_some_bins, 3)\n    #=> [\"\", -2, -1]","ref":"StreamData.html#frequency/1-examples"},{"type":"function","title":"Shrinking - StreamData.frequency/1","doc":"Each generated value is shrunk, and then this generator shrinks towards\nvalues generated by generators earlier in the list of `frequencies`.","ref":"StreamData.html#frequency/1-shrinking"},{"type":"function","title":"StreamData.integer/0","doc":"Generates integers bound by the generation size.","ref":"StreamData.html#integer/0"},{"type":"function","title":"Examples - StreamData.integer/0","doc":"Enum.take(StreamData.integer(), 3)\n    #=> [1, -1, -3]","ref":"StreamData.html#integer/0-examples"},{"type":"function","title":"Shrinking - StreamData.integer/0","doc":"Generated values shrink towards `0`.","ref":"StreamData.html#integer/0-shrinking"},{"type":"function","title":"StreamData.integer/1","doc":"Generates an integer in the given `range`.\n\nThe generation size is ignored since the integer always lies inside `range`.","ref":"StreamData.html#integer/1"},{"type":"function","title":"Examples - StreamData.integer/1","doc":"Enum.take(StreamData.integer(4..8), 3)\n    #=> [6, 7, 7]","ref":"StreamData.html#integer/1-examples"},{"type":"function","title":"Shrinking - StreamData.integer/1","doc":"Shrinks towards the smallest absolute value that still lie in `range`.","ref":"StreamData.html#integer/1-shrinking"},{"type":"function","title":"StreamData.iodata/0","doc":"Generates iodata.\n\nIodata are values of the `t:iodata/0` type.","ref":"StreamData.html#iodata/0"},{"type":"function","title":"Examples - StreamData.iodata/0","doc":"Enum.take(StreamData.iodata(), 3)\n    #=> [[\"\"], <<198>>, [115, 172]]","ref":"StreamData.html#iodata/0-examples"},{"type":"function","title":"Shrinking - StreamData.iodata/0","doc":"Shrinks towards less nested iodata and ultimately towards smaller binaries.","ref":"StreamData.html#iodata/0-shrinking"},{"type":"function","title":"StreamData.iolist/0","doc":"Generates iolists.\n\nIolists are values of the `t:iolist/0` type.","ref":"StreamData.html#iolist/0"},{"type":"function","title":"Examples - StreamData.iolist/0","doc":"Enum.take(StreamData.iolist(), 3)\n    #=> [[164 | \"\"], [225], [\"\" | \"\"]]","ref":"StreamData.html#iolist/0-examples"},{"type":"function","title":"Shrinking - StreamData.iolist/0","doc":"Shrinks towards smaller and less nested lists and towards bytes instead of\nbinaries.","ref":"StreamData.html#iolist/0-shrinking"},{"type":"function","title":"StreamData.keyword_of/1","doc":"Generates keyword lists where values are generated by `value_data`.\n\nKeys are always atoms.","ref":"StreamData.html#keyword_of/1"},{"type":"function","title":"Examples - StreamData.keyword_of/1","doc":"Enum.take(StreamData.keyword_of(StreamData.integer()), 3)\n    #=> [[], [sY: 1], [t: -1]]","ref":"StreamData.html#keyword_of/1-examples"},{"type":"function","title":"Shrinking - StreamData.keyword_of/1","doc":"This generator shrinks equivalently to a list of key-value tuples generated by\n`list_of/1`, that is, by shrinking the values in each tuple and also reducing\nthe size of the generated keyword list.","ref":"StreamData.html#keyword_of/1-shrinking"},{"type":"function","title":"StreamData.list_of/1","doc":"Generates lists where each values is generated by the given `data`.\n\nThe same as calling `list_of/2` with `[]` as options.","ref":"StreamData.html#list_of/1"},{"type":"function","title":"StreamData.list_of/2","doc":"Generates lists where each values is generated by the given `data`.\n\nEach generated list can contain duplicate elements. The length of the\ngenerated list is bound by the generation size. If the generation size is `0`,\nthe empty list will always be generated. Note that the accepted options\nprovide finer control over the size of the generated list. See the \"Options\"\nsection below.","ref":"StreamData.html#list_of/2"},{"type":"function","title":"Options - StreamData.list_of/2","doc":"* `:length` - (integer or range) if an integer, the exact length the\n    generated lists should be; if a range, the range in which the length of\n    the generated lists should be. If provided, `:min_length` and\n    `:max_length` are ignored.\n\n  * `:min_length` - (integer) the minimum length of the generated lists.\n\n  * `:max_length` - (integer) the maximum length of the generated lists.","ref":"StreamData.html#list_of/2-options"},{"type":"function","title":"Examples - StreamData.list_of/2","doc":"Enum.take(StreamData.list_of(StreamData.binary()), 3)\n    #=> [[\"\"], [], [\"\", \"w\"]]\n\n    Enum.take(StreamData.list_of(StreamData.integer(), length: 3), 3)\n    #=> [[0, 0, -1], [2, -1, 1], [0, 3, -3]]\n\n    Enum.take(StreamData.list_of(StreamData.integer(), max_length: 1), 3)\n    #=> [[1], [], []]","ref":"StreamData.html#list_of/2-examples"},{"type":"function","title":"Shrinking - StreamData.list_of/2","doc":"This generator shrinks by taking elements out of the generated list and also\nby shrinking the elements of the generated list. Shrinking still respects any\npossible length-related option: for example, if `:min_length` is provided, all\nshrunk list will have more than `:min_length` elements.","ref":"StreamData.html#list_of/2-shrinking"},{"type":"function","title":"StreamData.map/2","doc":"Maps the given function `fun` over the given generator `data`.\n\nReturns a new generator that returns elements from `data` after applying `fun`\nto them.","ref":"StreamData.html#map/2"},{"type":"function","title":"Examples - StreamData.map/2","doc":"iex> data = StreamData.map(StreamData.integer(), &Integer.to_string/1)\n    iex> Enum.take(data, 3)\n    [\"1\", \"0\", \"3\"]","ref":"StreamData.html#map/2-examples"},{"type":"function","title":"Shrinking - StreamData.map/2","doc":"This generator shrinks exactly like `data`, but with `fun` mapped over the\nshrunk data.","ref":"StreamData.html#map/2-shrinking"},{"type":"function","title":"StreamData.map_of/3","doc":"Generates maps with keys from `key_data` and values from `value_data`.\n\nSince maps require keys to be unique, this generator behaves similarly to\n`uniq_list_of/2`: if more than `max_tries` duplicate keys are generated\nconsequently, it raises a `StreamData.TooManyDuplicatesError` exception.","ref":"StreamData.html#map_of/3"},{"type":"function","title":"Options - StreamData.map_of/3","doc":"* `:length` - (non-negative integer) same as in `list_of/2`.\n\n  * `:min_length` - (non-negative integer) same as in `list_of/2`.\n\n  * `:max_length` - (non-negative integer) same as in `list_of/2`.","ref":"StreamData.html#map_of/3-options"},{"type":"function","title":"Examples - StreamData.map_of/3","doc":"Enum.take(StreamData.map_of(StreamData.integer(), StreamData.boolean()), 3)\n    #=> [%{}, %{1 => false}, %{-2 => true, -1 => false}]","ref":"StreamData.html#map_of/3-examples"},{"type":"function","title":"Shrinking - StreamData.map_of/3","doc":"Shrinks towards smallest maps and towards shrinking keys and values according\nto the respective generators.","ref":"StreamData.html#map_of/3-shrinking"},{"type":"function","title":"StreamData.mapset_of/2","doc":"Generates sets where values are generated by `data`.","ref":"StreamData.html#mapset_of/2"},{"type":"function","title":"Options - StreamData.mapset_of/2","doc":"* `:max_tries` - (non-negative integer) the maximum number of times that\n    this generator tries to generate the next element of the set before\n    giving up and raising a `StreamData.TooManyDuplicatesError` in case it\n    can't find a unique element to generate.","ref":"StreamData.html#mapset_of/2-options"},{"type":"function","title":"Examples - StreamData.mapset_of/2","doc":"Enum.take(StreamData.mapset_of(StreamData.integer()), 3)\n    #=> [#MapSet<[-1]>, #MapSet<[1, 2]>, #MapSet<[-3, 2, 3]>]","ref":"StreamData.html#mapset_of/2-examples"},{"type":"function","title":"Shrinking - StreamData.mapset_of/2","doc":"This generator shrinks in the same way as `uniq_list_of/2`, by removing\nelements and shrinking elements as well.","ref":"StreamData.html#mapset_of/2-shrinking"},{"type":"function","title":"StreamData.maybe_improper_list_of/2","doc":"Generates lists of elements out of `first` with a chance of them being\nimproper with the improper ending taken out of `improper`.\n\nBehaves similarly to `nonempty_improper_list_of/2` but can generate empty\nlists and proper lists as well.","ref":"StreamData.html#maybe_improper_list_of/2"},{"type":"function","title":"Examples - StreamData.maybe_improper_list_of/2","doc":"data = StreamData.maybe_improper_list_of(StreamData.byte(), StreamData.binary())\n    Enum.take(data, 3)\n    #=> [[60 | \".\"], [], [<<212>>]]","ref":"StreamData.html#maybe_improper_list_of/2-examples"},{"type":"function","title":"Shrinking - StreamData.maybe_improper_list_of/2","doc":"Shrinks towards smaller lists and shrunk elements in those lists, and\nultimately towards proper lists.","ref":"StreamData.html#maybe_improper_list_of/2-shrinking"},{"type":"function","title":"StreamData.member_of/1","doc":"Generates elements taken randomly out of `enum`.\n\n`enum` must be a non-empty and **finite** enumerable. If given an empty\nenumerable, this function raises an error. If given an infinite enumerable,\nthis function will not terminate.","ref":"StreamData.html#member_of/1"},{"type":"function","title":"Examples - StreamData.member_of/1","doc":"Enum.take(StreamData.member_of([:ok, 4, \"hello\"]), 3)\n    #=> [4, 4, \"hello\"]","ref":"StreamData.html#member_of/1-examples"},{"type":"function","title":"Shrinking - StreamData.member_of/1","doc":"This generator shrinks towards elements that appear earlier in `enum`.","ref":"StreamData.html#member_of/1-shrinking"},{"type":"function","title":"StreamData.non_negative_integer/0","doc":"Generates non-negative integers bound by the generation size.","ref":"StreamData.html#non_negative_integer/0"},{"type":"function","title":"Examples - StreamData.non_negative_integer/0","doc":"Enum.take(StreamData.non_negative_integer(), 3)\n    #=> [0, 2, 0]","ref":"StreamData.html#non_negative_integer/0-examples"},{"type":"function","title":"Shrinking - StreamData.non_negative_integer/0","doc":"Generated values shrink towards `0`.","ref":"StreamData.html#non_negative_integer/0-shrinking"},{"type":"function","title":"StreamData.nonempty/1","doc":"Constrains the given `enum_data` to be non-empty.\n\n`enum_data` must be a generator that emits enumerables, such as lists\nand maps. `nonempty/1` will filter out enumerables that are empty\n(`Enum.empty?/1` returns `true`).","ref":"StreamData.html#nonempty/1"},{"type":"function","title":"Examples - StreamData.nonempty/1","doc":"Enum.take(StreamData.nonempty(StreamData.list_of(StreamData.integer())), 3)\n    #=> [[1], [-1, 0], [2, 1, -2]]","ref":"StreamData.html#nonempty/1-examples"},{"type":"function","title":"StreamData.nonempty_improper_list_of/2","doc":"Generates non-empty improper lists where elements of the list are generated\nout of `first` and the improper ending out of `improper`.","ref":"StreamData.html#nonempty_improper_list_of/2"},{"type":"function","title":"Examples - StreamData.nonempty_improper_list_of/2","doc":"data = StreamData.nonempty_improper_list_of(StreamData.byte(), StreamData.binary())\n    Enum.take(data, 3)\n    #=> [[42], [56 | <<140, 137>>], [226 | \"j\"]]","ref":"StreamData.html#nonempty_improper_list_of/2-examples"},{"type":"function","title":"Shrinking - StreamData.nonempty_improper_list_of/2","doc":"Shrinks towards smaller lists (that are still non-empty, having the improper\nending) and towards shrunk elements of the list and a shrunk improper\nending.","ref":"StreamData.html#nonempty_improper_list_of/2-shrinking"},{"type":"function","title":"StreamData.one_of/1","doc":"Generates values out of one of the given `datas`.\n\n`datas` must be a list of generators. The values generated by this generator\nare values generated by generators in `datas`, chosen each time at random.","ref":"StreamData.html#one_of/1"},{"type":"function","title":"Examples - StreamData.one_of/1","doc":"data = StreamData.one_of([StreamData.integer(), StreamData.binary()])\n    Enum.take(data, 3)\n    #=> [-1, <<28>>, \"\"]","ref":"StreamData.html#one_of/1-examples"},{"type":"function","title":"Shrinking - StreamData.one_of/1","doc":"The generated value will be shrunk first according to the generator that\ngenerated it, and then this generator will shrink towards earlier generators\nin `datas`.","ref":"StreamData.html#one_of/1-shrinking"},{"type":"function","title":"StreamData.optional_map/2","doc":"Generates maps with fixed but optional keys and generated values.\n\n`data_map` is a map or keyword list of `fixed_key => data` pairs. Maps generated by this\ngenerator will have a subset of the keys of `data_map` and values corresponding to the values\ngenerated by the generator unders those keys.\n\nBy default, all keys are considered optional. A list of exactly which keys are optional can be\nprovided as the second argument, allowing for a map of mixed optional and required keys. The second argument is available since StreamData 0.6.0.\n\nSee also `fixed_map/1`.","ref":"StreamData.html#optional_map/2"},{"type":"function","title":"Examples - StreamData.optional_map/2","doc":"data = StreamData.optional_map(%{\n      integer: StreamData.integer(),\n      binary: StreamData.binary(),\n    })\n    Enum.take(data, 3)\n    #=> [%{binary: \"\", integer: 1}, %{integer: -2}, %{binary: \"R1^\"}]\n\n    data = StreamData.optional_map(%{\n      integer: StreamData.integer(),\n      binary: StreamData.binary(),\n    }, [:integer])\n    Enum.take(data, 3)\n    #=> [%{binary: \"\"}, %{binary: \"R1^\", integer: -2}, %{binary: \"R2^\"}]","ref":"StreamData.html#optional_map/2-examples"},{"type":"function","title":"Shrinking - StreamData.optional_map/2","doc":"This generator shrinks by first shrinking the map by taking out keys until the map is empty, and\nthen by shrinking the generated values.","ref":"StreamData.html#optional_map/2-shrinking"},{"type":"function","title":"StreamData.positive_integer/0","doc":"Generates positive integers bound by the generation size.","ref":"StreamData.html#positive_integer/0"},{"type":"function","title":"Examples - StreamData.positive_integer/0","doc":"Enum.take(StreamData.positive_integer(), 3)\n    #=> [1, 1, 3]","ref":"StreamData.html#positive_integer/0-examples"},{"type":"function","title":"Shrinking - StreamData.positive_integer/0","doc":"Generated values shrink towards `1`.","ref":"StreamData.html#positive_integer/0-shrinking"},{"type":"function","title":"StreamData.repeatedly/1","doc":"Calls the provided zero argument function to generate values.","ref":"StreamData.html#repeatedly/1"},{"type":"function","title":"Examples - StreamData.repeatedly/1","doc":"Generating a UUID\n\n    uuid = StreamData.repeatedly(&Ecto.UUID.generate/0)\n    Enum.take(uuid, 3)\n    #=> [\"2712ec5b-bc50-4b4a-8a8a-ca85d37a457b\", \"2092570d-8fb0-4e67-acbe-92db4c8a2bae\", \"1bef1fb1-8f86-46ac-a49e-3bffaa51e40b\"]\n\nGenerating a unique integer\n\n    integer = StreamData.repeatedly(&System.unique_integer([:positive, :monotonic]))\n    Enum.take(integer, 3)\n    #=> [1, 2, 3]","ref":"StreamData.html#repeatedly/1-examples"},{"type":"function","title":"Shrinking - StreamData.repeatedly/1","doc":"By nature, this generator is not shrinkable.","ref":"StreamData.html#repeatedly/1-shrinking"},{"type":"function","title":"StreamData.resize/2","doc":"Resize the given generated `data` to have fixed generation size `new_size`.\n\nThe new generator will ignore the generation size and always use `new_size`.\n\nSee the \"Generation size\" section in the documentation for `StreamData` for\nmore information about the generation size.","ref":"StreamData.html#resize/2"},{"type":"function","title":"Examples - StreamData.resize/2","doc":"data = StreamData.resize(StreamData.integer(), 10)\n    Enum.take(data, 3)\n    #=> [4, -5, -9]","ref":"StreamData.html#resize/2-examples"},{"type":"function","title":"StreamData.scale/2","doc":"Scales the generation size of the given generator `data` according to\n`size_changer`.\n\nWhen generating data from `data`, the generation size will be the result of\ncalling `size_changer` with the generation size as its argument. This is\nuseful, for example, when a generator needs to grow faster or slower than\nthe default.\n\nSee the \"Generation size\" section in the documentation for `StreamData` for\nmore information about the generation size.","ref":"StreamData.html#scale/2"},{"type":"function","title":"Examples - StreamData.scale/2","doc":"Let's create a generator that generates much smaller integers than `integer/0`\nwhen size grows. We can do this by scaling the generation size to the\nlogarithm of the generation size.\n\n    data = StreamData.scale(StreamData.integer(), fn size ->\n      trunc(:math.log(size))\n    end)\n\n    Enum.take(data, 3)\n    #=> [0, 0, -1]\n\nAnother interesting example is creating a generator with a fixed maximum\ngeneration size. For example, say we want to generate binaries but we never\nwant them to be larger than 64 bytes:\n\n    small_binaries = StreamData.scale(StreamData.binary(), fn size ->\n      min(size, 64)\n    end)","ref":"StreamData.html#scale/2-examples"},{"type":"function","title":"StreamData.seeded/2","doc":"Makes the given generator `data` always use the same given `seed` when generating.\n\nThis function is useful when you want a generator to have a predictable generating\nbehaviour. It's especially useful when using a generator with the `Enumerable` protocol\nsince you can't set the seed specifically in that case (while you can with `check_all/3`\nfor example).\n\n`seed` must be an integer.","ref":"StreamData.html#seeded/2"},{"type":"function","title":"Examples - StreamData.seeded/2","doc":"int = StreamData.seeded(StreamData.integer(), 10)\n\n    Enum.take(int, 3)\n    #=> [-1, -2, 1]\n    Enum.take(int, 4)\n    #=> [-1, -2, 1, 2]","ref":"StreamData.html#seeded/2-examples"},{"type":"function","title":"StreamData.sized/1","doc":"Returns the generator returned by calling `fun` with the generation size.\n\n`fun` takes the generation size and has to return a generator, that can use\nthat size to its advantage.\n\nSee the \"Generation size\" section in the documentation for `StreamData` for\nmore information about the generation size.","ref":"StreamData.html#sized/1"},{"type":"function","title":"Examples - StreamData.sized/1","doc":"Let's build a generator that generates integers in double the range `integer/0`\ndoes:\n\n    data = StreamData.sized(fn size ->\n      StreamData.resize(StreamData.integer(), size * 2)\n    end)\n\n    Enum.take(data, 3)\n    #=> [0, -1, 5]","ref":"StreamData.html#sized/1-examples"},{"type":"function","title":"StreamData.string/2","doc":"Generates a string of the given kind or from the given characters.\n\n`kind_or_codepoints` can be:\n\n  * `:ascii` - strings containing only ASCII characters are generated. Such\n    strings shrink towards lower codepoints.\n\n  * `:alphanumeric` - strings containing only alphanumeric characters\n    (`?a..?z`, `?A..?Z`, `?0..?9`) are generated. Such strings shrink towards\n    `?a` following the order shown previously.\n\n  * `:printable` - printable strings (`String.printable?/1` returns `true`)\n    are generated. Such strings shrink towards lower codepoints.\n\n  * `:utf8` - valid strings (`String.valid?/1` returns `true`)\n    are generated. Such strings shrink towards lower codepoints. *Available\n    since 0.6.0.*\n\n  * a range - strings with characters from the range are generated. Such\n    strings shrink towards characters that appear earlier in the range.\n\n  * a list of ranges or single codepoints - strings with characters from the\n    ranges or codepoints are generated. Such strings shrink towards earlier\n    elements of the given list and towards the beginning of ranges.","ref":"StreamData.html#string/2"},{"type":"function","title":"Options - StreamData.string/2","doc":"See the documentation of `list_of/2` for the possible values of options.","ref":"StreamData.html#string/2-options"},{"type":"function","title":"Examples - StreamData.string/2","doc":"Enum.take(StreamData.string(:ascii), 3)\n    #=> [\"c\", \"9A\", \"\"]\n\n    Enum.take(StreamData.string(Enum.concat([?a..?c, ?l..?o])), 3)\n    #=> [\"c\", \"oa\", \"lb\"]","ref":"StreamData.html#string/2-examples"},{"type":"function","title":"Shrinking - StreamData.string/2","doc":"Shrinks towards smaller strings and as described in the description of the\npossible values of `kind_or_codepoints` above.","ref":"StreamData.html#string/2-shrinking"},{"type":"opaque","title":"StreamData.t/1","doc":"An opaque type that represents a `StreamData` generator that generates values\nof type `a`.","ref":"StreamData.html#t:t/1"},{"type":"function","title":"StreamData.term/0","doc":"Generates any term.\n\nThe terms that this generator can generate are simple terms or compound terms. The simple terms\nare:\n\n  * integers (through `integer/0`)\n  * binaries (through `binary/1`)\n  * floats (through `float/0`)\n  * booleans (through `boolean/0`)\n  * atoms (through `atom/1`)\n  * references (which are not shrinkable)\n\nCompound terms are terms that contain other terms (which are generated recursively with\n`term/0`):\n\n  * lists (through `list_of/2`)\n  * maps (through `map_of/2`)\n  * tuples","ref":"StreamData.html#term/0"},{"type":"function","title":"Examples - StreamData.term/0","doc":"Enum.take(StreamData.term(), 3)\n    #=> [0.5119003572251588, {{true, \"\"}}, :WJg]","ref":"StreamData.html#term/0-examples"},{"type":"function","title":"Shrinking - StreamData.term/0","doc":"The terms generated by this generator shrink based on the generator used to create them (see the\nlist of possible generated terms above).","ref":"StreamData.html#term/0-shrinking"},{"type":"function","title":"StreamData.tree/2","doc":"Generates trees of values generated by `leaf_data` and `subtree_fun`.\n\n`leaf_data` generates the leaf nodes. `subtree_fun` is a function that is\ncalled by `tree`, if an inner node of the tree shall be generated. It takes a\ngenerator `child_gen` for child nodes and returns a generator for an inner\nnode using `child_gen` to go \"one level deeper\" in the tree. The frequency\nbetween leaves and inner nodes is 1:2.\n\nThis is best explained with an example. Say that we want to generate binary\ntrees of integers, and that we represent binary trees as either an integer (a\nleaf) or a `%Branch{}` struct:\n\n    defmodule Branch do\n      defstruct [:left, :right]\n    end\n\nNow, we can generate trees by using the `integer()` generator to generate\nthe leaf nodes. Then we can use the `subtree_fun` function to generate inner\nnodes (that is, `%Branch{}` structs or `integer()`s).\n\n    tree_data =\n      StreamData.tree(StreamData.integer(), fn child_data ->\n        StreamData.map({child_data, child_data}, fn {left, right} ->\n          %Branch{left: left, right: right}\n        end)\n      end)\n\n    Enum.at(StreamData.resize(tree_data, 10), 0)\n    #=> %Branch{left: %Branch{left: 4, right: -1}, right: -2}","ref":"StreamData.html#tree/2"},{"type":"function","title":"Examples - StreamData.tree/2","doc":"A common example is a nested list:\n\n    data = StreamData.tree(StreamData.integer(), &StreamData.list_of/1)\n    Enum.at(StreamData.resize(data, 10), 0)\n    #=> [[], '\\t', '\\a', [1, 2], -3, [-7, [10]]]\n\nA more complex example is generating data that could represent the Elixir\nequivalent of a JSON document. The code below is slightly simplified\ncompared to the JSON spec.\n\n    scalar_generator =\n      StreamData.one_of([\n        StreamData.integer(),\n        StreamData.boolean(),\n        StreamData.string(:ascii),\n        nil\n      ])\n\n    json_generator =\n      StreamData.tree(scalar_generator, fn nested_generator ->\n        StreamData.one_of([\n          StreamData.list_of(nested_generator),\n          StreamData.map_of(StreamData.string(:ascii, min_length: 1), nested_generator)\n        ])\n      end)\n\n    Enum.at(StreamData.resize(json_generator, 10), 0)\n    #=> [%{\"#\" => \"5\"}, true, %{\"4|B\" => nil, \"7\" => true, \"yt(3y\" => 4}, [[false]]]","ref":"StreamData.html#tree/2-examples"},{"type":"function","title":"Shrinking - StreamData.tree/2","doc":"Shrinks values and shrinks towards less deep trees.","ref":"StreamData.html#tree/2-shrinking"},{"type":"function","title":"StreamData.tuple/1","doc":"Generates tuples where each element is taken out of the corresponding\ngenerator in the `tuple_datas` tuple.","ref":"StreamData.html#tuple/1"},{"type":"function","title":"Examples - StreamData.tuple/1","doc":"data = StreamData.tuple({StreamData.integer(), StreamData.binary()})\n    Enum.take(data, 3)\n    #=> [{-1, <<170>>}, {1, \"<\"}, {1, \"\"}]","ref":"StreamData.html#tuple/1-examples"},{"type":"function","title":"Shrinking - StreamData.tuple/1","doc":"Shrinks by shrinking each element in the generated tuple according to the\ncorresponding generator.","ref":"StreamData.html#tuple/1-shrinking"},{"type":"function","title":"StreamData.uniq_list_of/2","doc":"Generates a list of elements generated by `data` without duplicates (possibly\naccording to a given uniqueness function).\n\nThis generator will generate lists where each list is unique according to the\nvalue returned by applying the given uniqueness function to each element\n(similarly to how `Enum.uniq_by/2` works). If more than the value of the\n`:max_tries` option consecutive elements are generated that are considered\nduplicates according to the uniqueness function, a\n`StreamData.TooManyDuplicatesError` error is raised. For this reason, try to\nmake sure to not make the uniqueness function return values out of a small\nvalue space. The uniqueness function and the max number of tries can be\ncustomized via options.","ref":"StreamData.html#uniq_list_of/2"},{"type":"function","title":"Options - StreamData.uniq_list_of/2","doc":"* `:uniq_fun` - (a function of arity one) a function that is called with\n    each generated element and whose return value is used as the value to\n    compare with other values for uniqueness (similarly to `Enum.uniq_by/2`).\n\n  * `:max_tries` - (non-negative integer) the maximum number of times that\n    this generator tries to generate the next element of the list before\n    giving up and raising a `StreamData.TooManyDuplicatesError` in case it\n    can't find a unique element to generate. Note that the generation size\n    often affects this: for example, if you have a generator like\n    `uniq_list_of(integer(), min_length: 4)` and you start generating elements\n    out of it with a generation size of `1`, it will fail by definition\n    because `integer/0` generates in `-size..size` so it would only generate\n    in a set (`[-1, 0, 1]`) with three elements. Use `resize/2` or `scale/2`\n    to manipulate the size (for example by setting a minimum generation size\n    of `3`) in such cases.\n\n  * `:length` - (non-negative integer) same as in `list_of/2`.\n\n  * `:min_length` - (non-negative integer) same as in `list_of/2`.\n\n  * `:max_length` - (non-negative integer) same as in `list_of/2`.","ref":"StreamData.html#uniq_list_of/2-options"},{"type":"function","title":"Examples - StreamData.uniq_list_of/2","doc":"data = StreamData.uniq_list_of(StreamData.integer())\n    Enum.take(data, 3)\n    #=> [[1], [], [2, 3, 1]]","ref":"StreamData.html#uniq_list_of/2-examples"},{"type":"function","title":"Shrinking - StreamData.uniq_list_of/2","doc":"This generator shrinks like `list_of/1`, but the shrunk values are unique\naccording to the `:uniq_fun` option as well.","ref":"StreamData.html#uniq_list_of/2-shrinking"},{"type":"function","title":"StreamData.unshrinkable/1","doc":"Makes the values generated by `data` not shrink.","ref":"StreamData.html#unshrinkable/1"},{"type":"function","title":"Examples - StreamData.unshrinkable/1","doc":"Let's build a generator of bytes (integers in the `0..255`) range. We can\nbuild this on top of `integer/1`, but for our purposes, it doesn't make sense for\na byte to shrink towards `0`:\n\n    byte = StreamData.unshrinkable(StreamData.integer(0..255))\n    Enum.take(byte, 3)\n    #=> [190, 181, 178]","ref":"StreamData.html#unshrinkable/1-examples"},{"type":"function","title":"Shrinking - StreamData.unshrinkable/1","doc":"The generator returned by `unshrinkable/1` generates the same values as `data`,\nbut such values will not shrink.","ref":"StreamData.html#unshrinkable/1-shrinking"},{"type":"exception","title":"StreamData.FilterTooNarrowError","doc":"","ref":"StreamData.FilterTooNarrowError.html"},{"type":"function","title":"StreamData.FilterTooNarrowError.message/1","doc":"","ref":"StreamData.FilterTooNarrowError.html#message/1"},{"type":"exception","title":"StreamData.TooManyDuplicatesError","doc":"","ref":"StreamData.TooManyDuplicatesError.html"},{"type":"function","title":"StreamData.TooManyDuplicatesError.message/1","doc":"","ref":"StreamData.TooManyDuplicatesError.html#message/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.0"}}